@@ -57,6 +57,7 @@ void parseirc(char *serveroutput)
 	char userhost[USERHOSTLEN + 1];
 	char nick[NICKLEN + 1];
 	char nick2[NICKLEN + 1];
+	char temp_nick[STRINGLEN + 1];
 	char target[NICKLEN + 1];
 	char m;
         char *restptr;
@@ -590,10 +591,13 @@ void parseirc(char *serveroutput)
 			return;
 		}
 		if(isme(nick)) {
-			/* no reason to remove from channel, we will just
-			   init it */
-			initchannel(this);
 			this->rejoin_at = mystate->now + OER_DELAY_BETWEEN_REJOINS;
+			if(this->nickcount == 1 && !this->i_am_op) {
+				/* we are going to cycle */
+				this->rejoin_at = mystate->now + OER_DELAY_BETWEEN_REJOINS_CYCLE;
+			}
+			/* no reason to remove from channel, we will just init it */
+			initchannel(this);
 			return;
 		}
 		addnewpart(this, mystate->now, nick, userhost);
@@ -701,8 +705,9 @@ void parseirc(char *serveroutput)
 		return;
 	}
 	if(!strcasecmp(command, "MODE")) {
+                /* NOTICE! Nick can also be a host (server/service modes) */
 		tpos = 0;
-		if((tpos = parse(prefix + 1, tpos, "!", nick, NICKLEN, 1)) < 0) {
+		if((tpos = parse(prefix + 1, tpos, "!", temp_nick, STRINGLEN, 1)) < 0) {
 			return;
 		}
 		strncpy(userhost, prefix + 1 + tpos, USERHOSTLEN);
@@ -724,15 +729,15 @@ void parseirc(char *serveroutput)
 		/* channel modes have no target */
 		if(strlen(restptr + restpos) > 0) {
 #ifdef DEBUG
-			oer_debug(OER_DEBUG_INFO, "parseirc(MODE)->%s %s on %s by %s!%s\n", mode, restptr + restpos, channel, nick, userhost);
+			oer_debug(OER_DEBUG_INFO, "parseirc(MODE)->%s %s on %s by %s!%s\n", mode, restptr + restpos, channel, temp_nick, userhost);
 #endif
 		} else {
 #ifdef DEBUG
-			oer_debug(OER_DEBUG_INFO, "parseirc(MODE)->%s on %s by %s!%s\n", mode, channel, nick, userhost);
+			oer_debug(OER_DEBUG_INFO, "parseirc(MODE)->%s on %s by %s!%s\n", mode, channel, temp_nick, userhost);
 #endif
 		}
 		/* lame way to keep chanmodes, i need to code a proper modeline parser */
-		if(!isme(nick) && (index(mode, (int)'p') != NULL || index(mode, (int)'s') != NULL || index(mode, (int)'i') != NULL || index(mode, (int)'t') != NULL || index(mode, (int)'n') != NULL || index(mode, (int)'m') != NULL || index(mode, (int)'l') != NULL || index(mode, (int)'k') != NULL)) {
+		if(!isme(temp_nick) && (index(mode, (int)'p') != NULL || index(mode, (int)'s') != NULL || index(mode, (int)'i') != NULL || index(mode, (int)'t') != NULL || index(mode, (int)'n') != NULL || index(mode, (int)'m') != NULL || index(mode, (int)'l') != NULL || index(mode, (int)'k') != NULL)) {
 			this->setchanmode = 1;			
 		}
 		i = 0;
@@ -758,12 +763,12 @@ void parseirc(char *serveroutput)
 					continue;
 				}
 				if(haschanflags(this, "u") && !isallowedop(this, target)) {
-					if(isq(nick) || isservice(nick)) {
+					if(isq(temp_nick) || isservice(temp_nick)) {
                                                 /* allow Q/service to set +o */
                                                 continue;
                                         }
 #ifdef DEBUG
-					oer_debug(OER_DEBUG_INFO, "parseirc(MODE)->non-allowed op of nick %s on channel %s by %s!%s\n", target, channel, nick, userhost);
+					oer_debug(OER_DEBUG_INFO, "parseirc(MODE)->non-allowed op of nick %s on channel %s by %s!%s\n", target, channel, temp_nick, userhost);
 #endif
 					mmode_new(this, mystate->now, "-o", target);
 				}
@@ -774,7 +779,7 @@ void parseirc(char *serveroutput)
 					this->i_am_op = 0;
 					continue;
 				}
-				if(isq(nick) || isservice(nick)) {
+				if(isq(temp_nick) || isservice(temp_nick)) {
 					/* allow Q/service to de-op */
 					continue;
 				}
@@ -796,14 +801,14 @@ void parseirc(char *serveroutput)
 #ifdef DEBUG
 					oer_debug(OER_DEBUG_INFO, "parseirc(MODE)->ban detected on me (%s)\n", target);
 #endif
-					if(isq(nick) || isservice(nick)) {
+					if(isq(temp_nick) || isservice(temp_nick)) {
                                                 /* allow Q/service to ban us */
                                                 continue;
                                         }
 					if(this->i_am_op && haschanflags(this, "b")) {
 						/* i am being banned, first kick then unban */
-						if(!isme(nick) && !isserver(nick)) {
-							snprintf(timed_str, WRITE_BUFFER_LENGTH, "KICK %s %s :do NOT ban %s, that's me!", channel, nick, target);
+						if(!isme(temp_nick) && !isserver(temp_nick)) {
+							snprintf(timed_str, WRITE_BUFFER_LENGTH, "KICK %s %s :do NOT ban %s, that's me!", channel, temp_nick, target);
 							timed_new(NULL, mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, timed_str);
 						}
 						mmode_new(this, mystate->now, "-b", target);
@@ -813,33 +818,33 @@ void parseirc(char *serveroutput)
 				break;
 			case OER_NTHMODE_UNBAN:
 				if(this->i_am_op && haschanflags(this, "e") && ispermban(this, target)) {
-					if(isq(nick) || isservice(nick)) {
+					if(isq(temp_nick) || isservice(temp_nick)) {
                                                 /* allow Q/service to unban */
                                                 continue;
                                         }
 					/* re-set the ban */
 #ifdef DEBUG
-					oer_debug(OER_DEBUG_INFO, "parseirc(MODE)->%s on channel %s tried to remove the permban %s\n", nick, channel, target);
+					oer_debug(OER_DEBUG_INFO, "parseirc(MODE)->%s on channel %s tried to remove the permban %s\n", temp_nick, channel, target);
 #endif
                                         mmode_new(this, mystate->now, "+b", target);
-					if(!isserver(nick)) {
+					if(!isserver(temp_nick)) {
 						snprintf(outstring, STRINGLEN, "do NOT remove %s ban from %s, it is in my permban list", target, channel);
-						sendreply(nick, 0, 3, 0, outstring);
+						sendreply(temp_nick, 0, 3, 0, outstring);
 					}
 				}
 				break;
 			}
-			if((deops > OER_MASSMODE_LIMIT) && (index(userhost, '@') != NULL) && !isme(nick)) {
+			if((deops > OER_MASSMODE_LIMIT) && (index(userhost, '@') != NULL) && !isme(temp_nick)) {
 #ifdef DEBUG
-				oer_debug(OER_DEBUG_INFO, "parseirc(MODE)->mass modes detected on %s from %s\n", channel, nick);
+				oer_debug(OER_DEBUG_INFO, "parseirc(MODE)->mass modes detected on %s from %s\n", channel, temp_nick);
 #endif
-				if(isq(nick) || isservice(nick)) {
+				if(isq(temp_nick) || isservice(temp_nick)) {
                                         /* allow Q/service to do massmodes */
                                         continue;
                                 }
-				if(this->i_am_op && haschanflags(this, "m") && !isserver(nick)) {
-                                        mmode_new(this, mystate->now, "-o", nick);
-					sendreply(nick, 0, 3, 0, "do NOT deop >3 users at a time");
+				if(this->i_am_op && haschanflags(this, "m") && !isserver(temp_nick)) {
+                                        mmode_new(this, mystate->now, "-o", temp_nick);
+					sendreply(temp_nick, 0, 3, 0, "do NOT deop >3 users at a time");
 					deops = 0;
 					bans = 0;
 				}
@@ -1476,7 +1481,7 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
                 }
 		if(!strlen(restptr)) {
 			/* the messager wants to be opped */
-			if(!isopnow(this, nick)) {
+			if(!isopnow(this, nick) && (haschanflags(this, "u") && isallowedop(this, nick))) {
 				mmode_new(this, mystate->now, "+o", nick);
 			}
 			return;
@@ -1489,10 +1494,7 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 				break;
 			}
 			ppos = nppos;
-			if(isopnow(this, p1)) {
-				continue;
-			}
-			if(haschanflags(this, "u") && isallowedop(this, p1)) {
+			if(!isopnow(this, p1) && (haschanflags(this, "u") && isallowedop(this, p1))) {
 				mmode_new(this, mystate->now, "+o", p1);
 			}
 		}
@@ -1515,7 +1517,7 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
                 }
 		if(!strlen(restptr)) {
 			/* the messager wants to be de-opped */
-			if(isopnow(this, nick)) {
+			if(isopnow(this, nick) && !(haschanflags(this, "D") && isallowedop(this, nick))) {
                                 mmode_new(this, mystate->now, "-o", nick);
 			}
 			return;
@@ -1528,7 +1530,7 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 				break;
 			}
 			ppos = nppos;
-			if(isopnow(this, p1)) {
+			if(isopnow(this, p1) && !(haschanflags(this, "D") && isallowedop(this, p1))) {
                                 mmode_new(this, mystate->now, "-o", p1);
 			}
 		}
