@@ -9,10 +9,9 @@
 #
 
 CREATE TABLE last_yourchannel (
-  twhen int(10) unsigned NOT NULL default '0',
-  nick varchar(32) NOT NULL default '',
-  hostmask varchar(64) NOT NULL default '',
-  message text NOT NULL,
-  KEY key1(nick)
-) TYPE=MyISAM;
-
+  twhen INT UNSIGNED NOT NULL,
+  nick VARCHAR(15) NOT NULL,
+  hostmask VARCHAR(80) NOT NULL,
+  message TEXT NOT NULL,
+  INDEX (nick)
+) TYPE=MyISAM PACK_KEYS=1;
